import 'package:flutter/material.dart';
import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'dart:convert';
import 'dart:io';

String host = 'https://app.nativenotify.com';

class NativeNotify {
  static void initialize(flutterAppId, flutterAppToken, firebaseServerKey) async {
    final prefs = await SharedPreferences.getInstance();

    await prefs.setInt('flutterAppId', flutterAppId);
    await prefs.setString('flutterAppToken', flutterAppToken);

    await Firebase.initializeApp();
    String? firebaseToken = await FirebaseMessaging.instance.getToken();
    print('firebase token: $firebaseToken');

    final flutterDeviceId = prefs.getInt('flutterDeviceId');

    if (flutterDeviceId == null) {
      try {
        var response = await http.post(Uri.parse('$host/api/flutter/device'),
            headers: {
              HttpHeaders.contentTypeHeader: 'application/json',
            },
            body: jsonEncode({
              'flutterAppId': '$flutterAppId',
              'flutterAppToken': flutterAppToken,
              'firebaseToken': firebaseToken,
              'firebaseServerKey': firebaseServerKey
            }));

        var jsonData = jsonDecode(response.body);

        await jsonData.forEach((key, value) async => {
              if (key == 'flutter_device_id')
                {await prefs.setInt('flutterDeviceId', value)}
            });

        print(
            "You can send Native Notify push notifications now. You're registered!");
      } catch (e) {
        // print(e);
      }
    } 

    AwesomeNotifications().initialize(null, [
      NotificationChannel(
          channelKey: 'mass',
          channelName: 'Native Notify Mass Notifications',
          channelDescription: "Notifications for all users",
          defaultColor: const Color(0XFF9050DD),
          ledColor: Colors.white,
          playSound: true,
          enableLights: true,
          enableVibration: true,
          importance: NotificationImportance.High)
    ]);

    AwesomeNotifications().isNotificationAllowed().then((isAllowed) {
      if (!isAllowed) {
        AwesomeNotifications().requestPermissionToSendNotifications();
      }
    });

    FirebaseMessaging.onMessage.listen(_firebaseMessagingHandler);
    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingHandler);
  }
}

Future<void> _firebaseMessagingHandler(RemoteMessage message) async {
  await Firebase.initializeApp();

  var deviceInfo = await jsonDecode(message.data['deviceInfo']);

  final flutterDeviceId = deviceInfo['flutterDeviceId'];
  final flutterAppId = deviceInfo['flutterAppId'];
  final flutterAppToken = deviceInfo['flutterAppToken'];

  try {
    await http.read(Uri.parse('$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));
  } catch (e) {
    // print(e);
  }

  AwesomeNotifications().createNotificationFromJsonData(message.data);
}
